import sys
import requests
from PySide6.QtWidgets import (
    QApplication,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QMessageBox,
)
from PySide6.QtCore import Qt

def get_weather_description(code: int) -> str:
    """Convert Open-Meteo weather code to a textual description."""
    if code == 0:
        return "Clear sky"
    elif code in (1, 2, 3):
        return "Mainly clear/partly cloudy/overcast"
    elif code in (45, 48):
        return "Fog"
    elif code in (51, 53, 55):
        return "Drizzle"
    elif code in (56, 57):
        return "Freezing Drizzle"
    elif code in (61, 63, 65):
        return "Rain"
    elif code in (66, 67):
        return "Freezing Rain"
    elif code in (71, 73, 75):
        return "Snow fall"
    elif code == 77:
        return "Snow grains"
    elif code in (80, 81, 82):
        return "Rain showers"
    elif code in (85, 86):
        return "Snow showers"
    elif code == 95:
        return "Thunderstorm"
    elif code in (96, 99):
        return "Thunderstorm with hail"
    else:
        return "Unknown weather"


class WeatherApp(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Weather Application")
        self.setup_ui()


    def setup_ui(self):
        layout = QVBoxLayout()

        # --- City Search Section ---
        city_layout = QHBoxLayout()
        self.city_input = QLineEdit()
        self.city_input.setPlaceholderText("Enter city name (e.g., Berlin)")
        city_layout.addWidget(self.city_input)

        self.search_button = QPushButton("Search City")
        self.search_button.clicked.connect(self.search_city)
        city_layout.addWidget(self.search_button)
        layout.addLayout(city_layout)

        # --- Coordinates Input Section ---
        coords_layout = QHBoxLayout()
        self.lat_input = QLineEdit()
        self.lat_input.setPlaceholderText("Latitude (e.g., 52.52)")
        coords_layout.addWidget(self.lat_input)

        self.lon_input = QLineEdit()
        self.lon_input.setPlaceholderText("Longitude (e.g., 13.41)")
        coords_layout.addWidget(self.lon_input)
        layout.addLayout(coords_layout)

        # --- Weather Fetch Button ---
        self.fetch_button = QPushButton("Get Weather")
        self.fetch_button.clicked.connect(self.fetch_weather)
        layout.addWidget(self.fetch_button)

        # --- Weather Display Section ---
        self.weather_label = QLabel("Weather information will appear here.")
        self.weather_label.setAlignment(Qt.AlignCenter)
        layout.addWidget(self.weather_label)

        self.setLayout(layout)


    def search_city(self):
        """
        Searches for the city coordinates using the Open-Meteo Geocoding API
        and fills in the latitude and longitude fields if a match is found.
        """
        city_name = self.city_input.text().strip()
        if not city_name:
            QMessageBox.warning(self, "Input Error", "Please enter a city name.")
            return

        geocoding_url = f"https://geocoding-api.open-meteo.com/v1/search?name={city_name}&count=1"
        try:
            response = requests.get(geocoding_url, timeout=10)
            response.raise_for_status()
            data = response.json()

            if "results" in data and data["results"]:
                result = data["results"][0]
                lat = result.get("latitude")
                lon = result.get("longitude")
                name = result.get("name")
                country = result.get("country")
                # Update the coordinate input fields.
                self.lat_input.setText(str(lat))
                self.lon_input.setText(str(lon))
                QMessageBox.information(
                    self,
                    "City Found",
                    f"Found {name}, {country}.\nCoordinates set to {lat}, {lon}.",
                )
            else:
                QMessageBox.information(self, "No Results", "No matching city was found.")
        except requests.RequestException as e:
            QMessageBox.critical(self, "Network Error", f"An error occurred: {e}")


    def fetch_weather(self):
        """
        Fetches the current weather for the provided coordinates using the Open-Meteo API.
        This call retrieves the following parameters from the current weather data:
          - temperature_2m
          - apparent_temperature
          - weather_code
          - surface_pressure
          - wind_speed_10m
          - wind_direction_10m
        """
        lat = self.lat_input.text().strip()
        lon = self.lon_input.text().strip()

        try:
            lat_val = float(lat)
            lon_val = float(lon)
        except ValueError:
            QMessageBox.warning(self, "Input Error", "Please enter valid numeric coordinates.")
            return

        # Construct the API URL with the specified 'current' parameters.
        url = (
            f"https://api.open-meteo.com/v1/forecast?"
            f"latitude={lat_val}&longitude={lon_val}"
            f"&current=temperature_2m,apparent_temperature,weather_code,surface_pressure,"
            f"wind_speed_10m,wind_direction_10m"
        )

        try:
            response = requests.get(url, timeout=10)
            response.raise_for_status()
            data = response.json()

            if "current" in data:
                current = data["current"]
                temperature_2m = current.get("temperature_2m")
                apparent_temperature = current.get("apparent_temperature")
                weather_code = current.get("weather_code")
                surface_pressure = current.get("surface_pressure")
                wind_speed = current.get("wind_speed_10m")
                wind_direction = current.get("wind_direction_10m")
                weather_time = current.get("time", "N/A")

                description = get_weather_description(weather_code)

                weather_info = (
                    f"Weather as of {weather_time}:\n"
                    f"Temperature (2 m): {temperature_2m} °C\n"
                    f"Apparent Temperature: {apparent_temperature} °C\n"
                    f"Surface Pressure: {surface_pressure} hPa\n"
                    f"Windspeed (10 m): {wind_speed} km/h\n"
                    f"Wind Direction (10 m): {wind_direction}°\n"
                    f"Weather Code: {weather_code} ({description})"
                )
            else:
                weather_info = "No current weather data available."

            self.weather_label.setText(weather_info)
        except requests.RequestException as e:
            QMessageBox.critical(self, "Network Error", f"An error occurred: {e}")
            self.weather_label.setText("Failed to retrieve weather data.")


def main():
    app = QApplication(sys.argv)
    window = WeatherApp()
    window.resize(500, 400)
    window.show()
    sys.exit(app.exec())


# Entry point of the application
if __name__ == "__main__":
    main()